<?php
namespace TrxEffects\Core;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Base class.
 *
 * A base class is used for creating other classes.
 *
 * @since 1.0.0
 */
class Base {

	/**
	 * Clone.
	 *
	 * Disable class cloning and throw an error on object clone.
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object. Therefore, we don't want the object to be cloned.
	 *
	 * @access public
	 * @since 1.0.0
	 */
	public function __clone() {
		_doing_it_wrong( __FUNCTION__, esc_html__( 'Can not clone this object.', 'trx-effects' ), '1.0.0' );
	}

	/**
	 * Wakeup.
	 *
	 * Disable unserializing of the class.
	 *
	 * @access public
	 * @since 1.0.0
	 */
	public function __wakeup() {
		_doing_it_wrong( __FUNCTION__, esc_html__( 'Can not unserialize this object.', 'trx-effects' ), '1.0.0' );
	}
}
