<?php
namespace TrxEffects\Core\Modules\BgCanvas;

use TrxEffects\Plugin;
use TrxEffects\Core\Modules\Module;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class BgCanvas extends Module {

	/**
	 * Elementor API object.
	 *
	 * Holds the object with Elementor API.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @var elementor
	 */
	public $elementor;


	/**
	 * Class constructor.
	 *
	 * Initializing module.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {
		add_action( 'trx_effects_action/load', array( $this, 'load' ), 20 );	// Need priority greater then 10 ( each next level of 'load' must be greater then caller class )
		add_action( 'trx_effects_action/init', array( $this, 'init' ) );
	}

	/**
	 * Load modules
	 *
	 * Create instance of modules
	 *
	 * @hook action 'trx_effects_action/load'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load() {
		parent::load();
		$this->elementor = new BgCanvasElementor();
	}


	/**
	 * Init module
	 *
	 * Do init actions of the module
	 *
	 * @hook action 'trx_effects_action/init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {
		parent::init();
		// Load scripts
		add_action( 'wp_enqueue_scripts', array( $this, 'load_scripts' ) );
	}


	/**
	 * Load scripts and styles
	 *
	 * Enqueue module-specific scripts and styles
	 *
	 * @hook action 'wp_enqueue_scripts'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load_scripts() {
		wp_enqueue_style(  'trx-effects-bg-canvas', Plugin::instance()->files->get_file_url( TRX_EFFECTS_MODULES . 'bg-canvas/bg-canvas.css' ), array(), null );
		wp_enqueue_script( 'trx-effects-bg-canvas', Plugin::instance()->files->get_file_url( TRX_EFFECTS_MODULES . 'bg-canvas/bg-canvas.js' ), array('jquery'), null, true );
	}

}
