<?php
namespace TrxEffects\Core\Modules\BgSlides;

use TrxEffects\Plugin;
use TrxEffects\Core\Modules\Module;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class BgSlidesElementor extends Module {

	/**
	 * Class constructor.
	 *
	 * Initializing module.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {
		add_action( 'trx_effects_action/init', array( $this, 'init' ) );
	}


	/**
	 * Init module
	 *
	 * Do init actions of the module
	 *
	 * @hook action 'trx_effects_action/init'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {

		// Add module-specific parameters to the params of the section
		add_action( 'elementor/element/before_section_start', array( $this, 'add_params_to_section' ), 10, 3 );

		// Add module-specific data-attributes to the section layout
		add_action( 'elementor/frontend/element/before_render', array( $this, 'add_data_attributes_to_section' ), 10, 1 );
		add_action( 'elementor/frontend/section/before_render', array( $this, 'add_data_attributes_to_section' ), 10, 1 );
	}


	/**
	 * Background slideshow parameters for sections
	 *
	 * Add module-specific parameters to the params of sections
	 *
	 * @hook action 'elementor/element/before_section_start'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_params_to_section( $element, $section_id, $args ) {

		if ( ! is_object( $element ) ) return;

		if ( in_array( $element->get_name(), array( 'section' ) ) && $section_id == '_section_responsive' ) {

			// Register controls
			$element->start_controls_section( 'section_trx_bg_slides', array(
				'tab' => ! empty( $args['tab'] ) ? $args['tab'] : \Elementor\Controls_Manager::TAB_ADVANCED,
				'label' => esc_html__( 'Background slideshow', 'trx-effects' )
			) );

			$element->add_control( 'bg_slides_allow', array(
				'label' => esc_html__( 'Allow slides', 'trx-effects' ),
				'label_on' => esc_html__( 'On', 'trx-effects' ),
				'label_off' => esc_html__( 'Off', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'return_value' => '1'
			) );

			$element->add_control( 'bg_slides_overlay_color', array(
				'label' => esc_html__( 'Overlay color', 'trx-effects' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::COLOR,
				'global' => array(
					'active' => false,
				),
				'default' => '',
				'selectors' => array(
					'{{WRAPPER}} .trx_effects_bg_slides_overlay' => 'background-color: {{VALUE}};',
				),
				'condition' => array(
					'bg_slides_allow' => '1'
				)
			) );

			$element->add_control( 'bg_slides_animation_duration', array(
				'label' => esc_html__( 'Animation duration', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'default' => array(
					'size' => 6.5,
					'unit' => 'px'
				),
				'range' => array(
					'px' => array(
						'min' => 0,
						'max' => 20,
						'step' => 0.1
					),
				),
				'size_units' => array( 'px' ),
				'condition' => array(
					'bg_slides_allow' => '1'
				),
			) );

			$repeater = new \Elementor\Repeater();

			$repeater->add_control( 'slide', array(
				'label' => esc_html__( 'Image', 'trx-effects' ),
				'type' => \Elementor\Controls_Manager::MEDIA,
				'default' => array(
					'url' => '',
				),
			) );

			$repeater->add_control( 'slide_size', array(
				'label' => esc_html__( 'Slide size', 'trx-effects' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => apply_filters( 'trx_effects_filter/bg_slides_sizes', array(
					'cover'   => esc_html__( 'Cover', 'trx-effects' ),
					'contain' => esc_html__( 'Contain', 'trx-effects' ),
					'fill'    => esc_html__( 'Fill', 'trx-effects' ),
				) ),
				'default' => 'cover',
			) );

			$repeater->add_control( 'slide_effect', array(
				'label' => esc_html__( 'Slide effect', 'trx-effects' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => apply_filters( 'trx_effects_filter/bg_slides_effects', array(
					'none'         => esc_html__( 'None', 'trx-effects' ),
					'zoom_in'      => esc_html__( 'Zoom In', 'trx-effects' ),
					'zoom_out'     => esc_html__( 'Zoom Out', 'trx-effects' ),
					'infinite_in'  => esc_html__( 'Infinite In', 'trx-effects' ),
					'infinite_out' => esc_html__( 'Infinite Out', 'trx-effects' ),
				) ),
				'default' => 'zoom_in'
			) );

			$repeater->add_control( 'slide_origin', array(
				'label' => esc_html__( 'Effect origin', 'trx-effects' ),
				'label_block' => false,
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => apply_filters( 'trx_effects_filter/bg_slides_origins', array(
					'lt' => esc_html__( 'Top Left', 'trx-effects' ),
					'ct' => esc_html__( 'Top Center', 'trx-effects' ),
					'rt' => esc_html__( 'Top Right', 'trx-effects' ),
					'lc' => esc_html__( 'Middle Left', 'trx-effects' ),
					'cc' => esc_html__( 'Middle Center', 'trx-effects' ),
					'rc' => esc_html__( 'Middle Right', 'trx-effects' ),
					'lb' => esc_html__( 'Bottom Left', 'trx-effects' ),
					'cb' => esc_html__( 'Bottom Center', 'trx-effects' ),
					'rb' => esc_html__( 'Bottom Right', 'trx-effects' ),
				) ),
				'default' => 'cc',
				'condition' => array(
					'slide_effect!' => 'none'
				)
			) );

			$element->add_control( 'bg_slides', array(
				'type' => \Elementor\Controls_Manager::REPEATER,
				'label' => esc_html__( 'Images', 'trx-effects' ),
				'fields' => $repeater->get_controls(),
				'title_field' => '{{{ slide_effect }}}',
				'condition' => array(
					'bg_slides_allow' => '1'
				)
			) );


			$element->end_controls_section();
		}
	}


	/**
	 * Add "data-trx-effects-bg-text"  to the wrapper of the element
	 *
	 * Hook render elements and add data parameters to its layout
	 *
	 * @hook action 'elementor/frontend/element/before_render'
	 * @hook action 'elementor/frontend/widget/before_render'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function add_data_attributes_to_section( $element ) {
		if ( is_object( $element ) && in_array( $element->get_name(), array( 'section' ) ) ) {
			$settings = $element->get_settings();
			if ( (int) $settings['bg_slides_allow'] > 0 && ! empty( $settings['bg_slides'][0]['slide']['url'] ) ) {
				$element->add_render_attribute( '_wrapper', 'class', 'trx_effects_has_bg_slides' );
				$element->add_render_attribute( '_wrapper', 'data-trx-effects-bg-slides', json_encode( array(
					'bg_slides_allow'              => (int) $settings['bg_slides_allow'],
					'bg_slides'                    => $settings['bg_slides'],
					'bg_slides_overlay_color'      => $settings['bg_slides_overlay_color'],
					'bg_slides_animation_duration' => $settings['bg_slides_animation_duration']['size'],
					)
				) );
			}
		}
	}

}
