<?php
namespace TrxEffects\Core\Modules;

use TrxEffects\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

require_once TRX_EFFECTS_DIR . 'core/modules/module.php';

class Manager {

	/**
	 * Modules
	 *
	 * Holds objects of the modules.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var bg_canvas
	 */
	private $modules = array();


	/**
	 * Class constructor.
	 *
	 * Initializing modules manager.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {
		add_action( 'trx_effects_action/load', array( $this, 'load') );
	}


	/**
	 * Load modules
	 *
	 * Create instance of modules
	 *
	 * @hook action 'trx_effects_action/load'
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function load() {
		foreach( $this->get_list_modules() as $slug => $title ) {
			if ( (int) Plugin::instance()->options->get_option( "modules[{$slug}]" ) > 0 ) {
				$class_name = join( '', array_map( 'ucwords', explode( '-', $slug ) ) );
				$class_path = "TrxEffects\\Core\\Modules\\{$class_name}\\{$class_name}";
				$this->modules[ $slug ] = new $class_path();
			}
		}
	}


	/**
	 * Available modules
	 *
	 * Return a list of available modules
	 *
	 * @return (array) list
	 * 
	 * @since 1.0.0
	 * @access public
	 */
	public function get_list_modules() {
		return array(
					"mouse-helper"  => esc_html__( 'Mouse helper', 'trx-effects' ),
					"image-effects" => esc_html__( 'Image effects', 'trx-effects' ),
					"image-mask"    => esc_html__( 'Image mask', 'trx-effects' ),
					"bg-canvas"     => esc_html__( 'Dynamic background', 'trx-effects' ),
					"bg-text"       => esc_html__( 'Background text', 'trx-effects' ),
					"bg-slides"     => esc_html__( 'Background slides', 'trx-effects' ),
				);
	}
}
