<?php
namespace TrxEffects\Core\Options;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * An option field of the type 'icons'
 *
 * Add a icons selector to the options page
 *
 * @since 1.0.0
 */
class CustomizerFieldIcons extends CustomizerField {

	/**
	 * Display an inner layout of the field.
	 *
	 * Display tags inside field's wrapper.
	 *
	 * @param (string) name - a name of the field
	 * @param (array) field - an array with field description
	 *
	 * @return (string) output - a string with a html layout of the field
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function show_field_layout( $id, $name, $field ) {
		$field['options'] = apply_filters( 'trx_effects_filter/options_get_list_choises', $field['options'], $name );
		$output = '<input type="hidden" name="trx_effects_options_field_' . esc_attr( $name ) . '"'
						. ' value="' . esc_attr( $field['val'] ) . '"'
					. ' />';
		if ( is_array( $field['options'] ) && count( $field['options'] ) > 0 ) {
			if ( empty( $field['style'] ) )  $field['style']  = 'icons';
			if ( empty( $field['return'] ) ) $field['return'] = 'full';
			if ( empty( $field['mode'] ) )   $field['mode']   = 'dropdown';
			$output .= ( $field['mode'] == 'dropdown'
							? '<span class="trx_effects_icon_selector'
											. ( ! empty( $field['class_field'] ) ? ' ' . esc_attr( $field['class_field'] ) : '' ) 
											. ( $field['style'] == 'icons' && ! empty( $field['val'] ) ? ' ' . esc_attr( $field['val'] ) : '' )
											. '"'
									. ' title="' . esc_attr__( 'Select icon', 'trx-effects' ) . '"'
									. ' data-style="' . esc_attr( $field['style'] ) . '"'
									. ( in_array( $field['style'], array( 'images', 'svg' ) ) && ! empty( $field['val'] )
											? ' style="background-image: url(' . esc_url( $field['return'] == 'slug' 
																							? $field['options'][ $field['val'] ] 
																							: $value
																						)
																				. ');"' 
											: '' )
								. '></span>'
							: ''
							)
						. '<div class="trx_effects_list_icons trx_effects_list_icons_' . esc_attr( $field['mode'] ) . '">'
						. ( $field['mode'] == 'dropdown'
							? '<input type="text" class="trx_effects_list_icons_search" placeholder="' . esc_attr__( 'Search icon ...', 'trx-effects' ) . '">'
							: ''
							);
			foreach ( $field['options'] as $slug => $icon ) {
				$output .= '<span class="' . esc_attr( $field['style'] == 'icons' ? $icon : $slug )
										. ( ( $field['return'] == 'full' ? $icon : $slug ) == $field['val']
											? ' trx_effects_active'
											: ''
											)
									. '"'
									. ' title="' . esc_attr( $slug ) . '"'
									. ' data-icon="' . esc_attr( $field['return'] == 'full' ? $icon : $slug ) . '"'
									. ( in_array( $field['style'], array( 'images', 'svg' ) )
										? ' style="background-image: url(' . esc_url( $icon ) . ');"'
										: ''
										)
							. '>'
								. ( $field['mode'] != 'dropdown' ? '<i>' . esc_html( $slug ) . '</i>' : '' )
							. '</span>';
			}
			$output .= '</div>';
		}
		return $output;
	}

}
