<?php
namespace TrxEffects\Core\Options;

use TrxEffects\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * An option field of the type 'media'
 *
 * Add a media to the options page
 *
 * @since 1.0.0
 */
class CustomizerFieldMedia extends CustomizerField {

	/**
	 * Display an inner layout of the field.
	 *
	 * Display tags inside field's wrapper.
	 *
	 * @param (string) name - a name of the field
	 * @param (array) field - an array with field description
	 *
	 * @return (string) output - a string with a html layout of the field
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function show_field_layout( $id, $name, $field ) {

		wp_enqueue_media( );

		$title = ! empty( $field['button_caption'] ) 
					? $field['button_caption']
					: ( empty( $field['type'] ) || $field['type'] == 'image'
						? esc_html__( 'Choose Image', 'trx-effects')
						: esc_html__( 'Choose Media', 'trx-effects')
						);

		$output = ( ! empty( $field['multiple'] ) 
					? '<input type="hidden" id="trx_effects_options_field_' . esc_attr( $id ) . '"'
						. ' name="trx_effects_options_field_' . esc_attr( $name ) . '"'
						. ' value="' . esc_attr( $field['val'] ) . '"'
						. '>'
					: '<input type="text" id="trx_effects_options_field_' . esc_attr( $id ) . '"'
						. ' name="trx_effects_options_field_' . esc_attr( $name ) . '"'
						. ' value="' . esc_attr( $field['val'] ) . '"'
						. ' />'
					)
					. '<input type="button"'
						. ' id="trx_effects_options_field_' . esc_attr( $id ) . '_button"'
						. ' class="button mediamanager trx_effects_media_selector'
							. ( ! empty( $field['class_field'] ) ? ' ' . esc_attr( $field['class_field'] ) : '' ) 
							. '"'
						. ' data-choose="' . esc_attr( ! empty( $field['multiple'] ) ? esc_html__( 'Choose Images', 'trx-effects' ) : $title ) . '"'
						. '	data-update="' . esc_attr( ! empty( $field['multiple'] ) ? esc_html__( 'Add to Gallery', 'trx-effects' ) : $title ) . '"'
						. ' data-multiple="' . esc_attr( ! empty( $field['multiple'] ) ? '1' : '0' ) . '"'
						. ' data-type="' . esc_attr( ! empty( $field['type'] ) ? $field['type'] : 'image' ) . '"'
						. ' data-linked-field="trx_effects_options_field_' . esc_attr( $id ) . '"'
						. ' value="'
							. ( ! empty( $field['multiple'] )
								? ( empty( $field['type'] ) || $field['type'] == 'image'
									? esc_attr__( 'Add Images', 'trx-effects' )
									: esc_attr__( 'Add Files', 'trx-effects' )
									)
								: esc_attr( $title )
								)
							. '"'
						. '>';
		$images = explode( '|', $field['val'] );
		$output .= '<span class="trx_effects_media_selector_preview">';
		if ( is_array( $images ) ) {
			foreach ( $images as $img ) {
				$output .= $img
					? '<span>'
							. ( in_array( Plugin::instance()->files->get_file_ext( $img ), array( 'gif', 'jpg', 'jpeg', 'png' ) )
									? '<img src="' . esc_url( $img ) . '" alt="' . esc_attr__( 'Selected image', 'trx-effects') . '">'
									: '<a href="' . esc_attr( $img ) . '">' . esc_html( basename( $img ) ) . '</a>'
								)
						. '</span>' 
					: '';
			}
		}
		$output .= '</span>';
		return $output;
	}

}
